/*Original code from Frederic Hecht <hecht@ann.jussieu.fr> (BAMG v1.01, QuadTree.h)*/
#ifndef _QUADTREE_H
#define _QUADTREE_H

#include "../../shared/shared.h"
#include "../../include/macros.h"
#include "../../toolkits/toolkits.h"


namespace bamg {

	typedef long IntQuad;

	const int MaxDeep = 30;
	const IntQuad MaxISize = ( 1L << MaxDeep); 

	class Triangles;
	class Vertex;

	class QuadTree{
		private:
			class QuadTreeBox { 
				public:
					long n; 
					//contains only one object form the list (either Vertex or QuadTreeBox)
					// if n < 4 => Vertex else =>  QuadTreeBox;
					union{
						QuadTreeBox* b[4];
						Vertex* v[4];
					};
			};
			class StorageQuadTreeBox {
				public:
					QuadTreeBox *b,*bc,*be;
					long len;
					StorageQuadTreeBox* n; // next StorageQuadTreeBox
					StorageQuadTreeBox(long ,StorageQuadTreeBox* =NULL);
					~StorageQuadTreeBox() {
						if(n) delete n;
						delete [] b;
					}
					long  SizeOf() const {return len*sizeof(QuadTreeBox)+sizeof(StorageQuadTreeBox)+ (n?n->SizeOf():0);}
			};
			StorageQuadTreeBox* sb;
			long                lenStorageQuadTreeBox;

		public:
			//fields
			QuadTreeBox* root;
			Triangles*   th;
			//functions
			~QuadTree();
			QuadTree(Triangles * t,long nbv=-1);
			QuadTree();
			long    NbQuadTreeBox,NbVertices;
			long    NbQuadTreeBoxSearch,NbVerticesSearch;
			Vertex* NearestVertex(Icoor1 i,Icoor1 j);
			Vertex* NearestVertexWithNormal(Icoor1 i,Icoor1 j);
			Vertex* ToClose(Vertex & ,double ,Icoor1,Icoor1);
			long    SizeOf() const {return sizeof(QuadTree)+sb->SizeOf();}
			void    Add( Vertex & w);
			QuadTreeBox* NewQuadTreeBox(){
				if(! (sb->bc<sb->be)) sb=new StorageQuadTreeBox(lenStorageQuadTreeBox,sb);
				if (!sb || (sb->bc->n != 0)){throw ErrorException(__FUNCT__,exprintf("!sb || (sb->bc->n != 0)"));}
				NbQuadTreeBox++;
				return sb->bc++;
			}
	};
}
#endif
